﻿-- Author      : Potdisc
-- Create Date : 3/21/2012 3:46:51 PM
-- Mainframe.lua Handles all the masterloot interaction and host/client comms

--[[_____________________________.
-- TODO: 	
-- Build a "Loot History Explorer"	
-- Add "notes" to the loot frame.
-- Possibility for swicthing between items as councillor.

--_____________________________.]]
--[[ CHANGELOG
	==== 1.3.2
	
	*++Added option to autoloot BoE.++
	*You can now choose if the autolooter should loot BoE (Bind on Equip) items or not.
	 
	*Alt click looting now works with XLoot 1.0 and ElvUI.
	
	Bugfixes:
	*//Fixed serveral things causing errors.//
]]


RCLootCouncil = LibStub("AceAddon-3.0"):NewAddon("RCLootCouncil", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0", "AceSerializer-3.0", "AceHook-3.0");
RCLootCouncil:SetDefaultModuleLibraries("AceEvent-3.0")
RCLootCouncil:SetDefaultModuleState(false)

RCLootCouncil_Mainframe = {}
RCLootCouncil_LootFrame = {}
RCLootCouncil_RankFrame = {}

local debug = false -- enable printing of debugging messages
local nnp = false
local superDebug = false -- extra debugging (very spammy)
local version = '1.3.2'

local entryTable = {{},{},{},{},{},{},{},{},{},{}};
-- entryTable order: (playerName, rank, role, totalIlvl, response, gear1, gear2, votes, class, color[], haveVoted, voters[])

local isMasterLooter = false; -- is the player master looter?
local isCouncil = false; -- is the player in the council?
local isRunning = false; -- should we use the addon?
local masterLooter = ""; -- name of master looter
local currentCouncil = {} -- The current council recieved from the masterlooter
local itemRunning = nil; -- is there an item being looted?
local guildRank = ""; -- Player's rank in guild
local selection = {}; -- The current selection
local lootNum = 0; -- the number of GetNumLootItems we've reached e.g. which item we're at
local itemsToLootIndex = {}; -- table containing the GetLootSlotLink() indexes that needs to be looted
local lootTable = {} -- table containing the actual items
local hasVerCheck = false; -- used to prevent "please upgrade" spamming
local bossName = "" -- used to get the boss name

local MAX_DISPLAY = 11;
local offset = 0; -- scrollframe offset
local db, buttonsDB, lootDB; -- shortening of self.db.profile(.buttons/.lootDB)
local mlDB; -- the master looter's db

local sortMethod = 'desc';
local currSortIndex = 0;
local self = RCLootCouncil;
local guildEventUnregister = false; -- used to unregister the guild_roster_update event
local _; -- Undo Blizzard's powertainting!

-- Create the defaults
local defaults = {
		factionrealm = { -- the loot awarded db should be the same for all characters in the same faction on the same realm
			lootDB = {
				--[[ Format:
				"playerName" = {
					[#] = {"lootWon", "date (d/m/y)", "time (h:m:s)", "instance", "boss", "votes", "itemReplaced1", "itemReplaced2", "response", "responseID"}
				},
				]]
			},
		},
		profile = {
			council = {},
			selfVote = true,
			multiVote = true,
			awardMessageChat1 = "RAID",
			awardMessageText1 = "&p was awarded with &i!",
			awardMessageChat2 = "NONE",
			awardMessageText2 = "",
			anonymousVoting = false,
			masterLooterOnly = false,
			autoLooting = true,
			altClickLooting = true,
			lootEverything = true,
			acceptWhispers = true,
			trackAwards = true,
			advancedOptions = false,
			autolootBoE = false,
			minRank = -1,
			numButtons = 4,
			maxButtons = 8,			
			buttons = {
				{	--1
					text = "Mainspec/Need",
					color = {0, 1, 0,1}, -- Green
					response = "Mainspec/Need"
				},
				{	--2
					text = "Offspec/greed",
					color = {1, 0.5, 0,1}, -- Orange
					response = "Offspec/Greed"
				},
				{	--3
					text = "Minor Upgrade",
					color = {0, 0.75, 0.75,1}, -- LightBlue
					response = "Minor Upgrade"
				},
				{	--4
					text = "Pass",
					color = {0.75, 0.75,0.75,1}, -- Gray
					response = "Pass"
				},
			},
		},
	}
	-- create the "unnormal" buttons
	for i = 5, defaults.profile.maxButtons do
		defaults.profile.buttons[i] = {
			text = "Button "..i,
			color = {0.75, 0.75,0.75,1},
			response = "Button "..i.." response",
		}
	end

function RCLootCouncil:OnInitialize()
	self:RegisterChatCommand("rc", "ChatCommand")
    self:RegisterChatCommand("rclc", "ChatCommand")
	self:RegisterComm("RCLootCouncil")
	self.db = LibStub("AceDB-3.0"):New("RCLootCouncilDB", defaults, true)
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	db = self.db.profile
	lootDB = self.db.factionrealm.lootDB

	self.options = self:OptionsTable()
	-- register the optionstable
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RCLootCouncil", self.options)
	
	-- add it to blizz options
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RCLootCouncil", "RCLootCouncil")	
	buttonsDB = db.buttons
end

function RCLootCouncil:OnEnable()  
	MainFrame:RegisterEvent("LOOT_OPENED");
	MainFrame:RegisterEvent("LOOT_CLOSED");
	MainFrame:RegisterEvent("GROUP_ROSTER_UPDATE");
	MainFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
	MainFrame:RegisterEvent("GUILD_ROSTER_UPDATE");
	MainFrame:RegisterEvent("CHAT_MSG_WHISPER");
	MainFrame:SetScript("OnEvent", RCLootCouncil.EventHandler);

	if IsInGuild() then
		self:SendCommMessage("RCLootCouncil", "verTest "..version, "GUILD") -- send out a version check
	end
	GuildRoster();	
	RCLootCouncil_LootFrame:OnLoad()
	
end

function RCLootCouncil:OnDisable()
	self:UnregisterAllEvents()
	MainFrame:UnregisterAllEvents()
end

function RCLootCouncil:RefreshConfig()
	db = self.db.profile
end

------------- MainFrame_OnLoad -----------
-- Loads the MainFrame
------------------------------------------
function RCLootCouncil:MainFrame_OnLoad()
	MainFrame:Hide()
	------ Create content entries --------
	local entry = CreateFrame("Button", "$parentEntry1", ContentFrame, "RCLootCouncil_Entry"); -- Creates the first row
	entry:SetID(1); -- Set the ID
	entry:SetPoint("TOPLEFT", 4, -4) -- Set anchor
	for i = 2, MAX_DISPLAY do -- Create the rest of the rows
		local entry = CreateFrame("Button", "$parentEntry"..i, ContentFrame, "RCLootCouncil_Entry");
		entry:SetID(i);
		entry:SetPoint("TOP", "$parentEntry"..(i-1), "BOTTOM") -- Set the anchor to the row above
	end
	----------PopUp setups --------------
	-------------------------------------
	StaticPopupDialogs["RCLOOTCOUNCIL_CONFIRM_ABORT"] = {
		text = "Are you sure you want to abort?",
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			RCLootCouncil_Mainframe.abortLooting()
			self:SendCommMessage("RCLootCouncil", "stop", "RAID") -- tell the council to abort aswell
			RCLootCouncil_Mainframe.stopLooting()
			CloseButton_OnClick() -- close the frame
			CloseLoot() -- close the lootlist
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3, 
	}

	StaticPopupDialogs["RCLOOTCOUNCIL_CONFIRM_AWARD"] = {
		text = "Are you sure you want to give %s to %s?",
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			RCLootCouncil_Mainframe.award()
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3, 
	}

	StaticPopupDialogs["RCLOOTCOUNCIL_CONFIRM_USAGE"] = {
		text = "Do you want to use RCLootCouncil for this raid?",
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			isRunning = true;
			masterLooter = RCLootCouncil_Mainframe.getML()
			self:Print("is active in this raid. You can turn it off in the options menu if you regret.")
			if #db.council < 1 then -- if there's no council
				self:Print("You've not set a council")
				RCLootCouncil_RankFrame.show() -- show the rankframe
			end
		end,
		OnCancel = function ()
			isRunning = false;
			self:Print("is not active in this raid. You can turn it on in the options menu if you regret.")
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3, 
	} 
end

---------- EventHandler -----------------
-- Handles events
-----------------------------------------
function RCLootCouncil.EventHandler(self2, event, ...)
	if event == "LOOT_OPENED" then
		if isMasterLooter and isRunning and (IsInRaid() or nnp) then -- if we're masterlooter and addon is enabled or debug is on
			if GetNumLootItems() >= 1 then -- if there's something to loot
				if not InCombatLockdown() then
					bossName = GetUnitName("target") -- extract the boss name before the player can switch targets
					for i = 1, GetNumLootItems() do -- run through the number of items in the loot window
						if db.altClickLooting then -- hook loot buttons if we use altClickLooting
							self:HookScript(getglobal("LootButton"..i), "OnClick", "LootOnClick") -- hook if we use it
							if XLoot then -- hook XLoot
								self:HookScript(getglobal("XLootButton"..i), "OnClick", "LootOnClick")
							end if XLootFrame then -- if XLoot 1.0
								self:HookScript(getglobal("XLootFrameButton"..i), "OnClick", "LootOnClick")
							end if getglobal("ElvLootSlot"..i) then -- if ElvUI
								self:HookScript(getglobal("ElvLootSlot"..i), "OnClick", "LootOnClick")
							end
						end
						if db.autoLooting then -- don't go further unless we want to autoloot
							local _, _, lootQuantity, lootRarity = GetLootSlotInfo(i)
							if (LootSlotHasItem(i) or db.lootEverything) and lootQuantity > 0 and lootRarity >= GetLootThreshold() then -- Check wether we want to loot the item or not
								-- now that we now it's an lootable item, lets also check if we should loot BoE's
								if RCLootCouncil:LootBoE(GetLootSlotLink(i)) then
									tinsert(lootTable, GetLootSlotLink(i)) -- add the item link to the table
									tinsert(itemsToLootIndex, i) -- add its index to the lootTable
								end
							elseif lootQuantity == 0 and db.lootEverything then -- if its coin
								LootSlot(i) -- loot the coin
							end
						end
					end
				else -- don't do anything if we're in combat
					self:Print("Couldn't start the loot session because you're in combat.")
					return;
				end
				if #itemsToLootIndex > 0 then -- if there's anything in the table
					mlDB = db -- create the master looter db
					self:SendCommMessage("RCLootCouncil", "mlDB "..self:Serialize(mlDB), "RAID") -- and send it
					RCLootCouncil_LootFrame.setupLoot(lootTable) -- setup our own loot frames
					self:SendCommMessage("RCLootCouncil", "lootTable "..self:Serialize(lootTable), "RAID") -- tell everyone to do the same
					lootNum = 1;
					RCLootCouncil_initiateLoot(GetLootSlotLink(itemsToLootIndex[lootNum]), GetUnitName("player")); -- initiate the loot
				end
			end
		end
	elseif event == "LOOT_CLOSED" then
		self:UnhookAll()
		if itemRunning then -- the loot window closed too early
			DEFAULT_CHAT_FRAME:AddMessage("RCLootCouncil cannot loot anything without the loot window being open!", 1, 0.5, 0, 1, 10)
			DEFAULT_CHAT_FRAME:AddMessage("Please restart the looting session.", 1, 0.5, 0, 1, 10)
			RCLootCouncil_Mainframe.abortLooting()
			RCLootCouncil_Mainframe.stopLooting()
			CloseButton_OnClick() -- close the frame
			self:SendCommMessage("RCLootCouncil", "stop", "RAID") -- tell the council to abort aswell
		end

	elseif event == "GROUP_ROSTER_UPDATE" and IsInRaid() then
		masterLooter = RCLootCouncil_Mainframe.getML()
		isCouncil = RCLootCouncil_Mainframe.isCouncil()
		
	elseif event == ("PLAYER_ENTERING_WORLD") then
		if IsInRaid() then -- if the player is in a raid
			local lootMethod, _, MLRaidID = GetLootMethod()
			local inInstance, instanceType = IsInInstance()
			if lootMethod == 'master' then -- if master looter is turned on
				local name  = GetRaidRosterInfo(MLRaidID)
				if name == GetUnitName("player") and inInstance and instanceType == "raid" and not isRunning then -- then ask him if he wants to use the addon
					StaticPopup_Show("RCLOOTCOUNCIL_CONFIRM_USAGE")
				end
			elseif inInstance and instanceType == "raid" and UnitIsGroupLeader("player") and not isRunning then -- otherwise, ask the raid leader
				StaticPopup_Show("RCLOOTCOUNCIL_CONFIRM_USAGE")
			end
		end
	elseif event == "GUILD_ROSTER_UPDATE" then -- delay the getting of guildRank till it's available
		guildRank = RCLootCouncil:GetGuildRank();
		if guildEventUnregister then
			MainFrame:UnregisterEvent("GUILD_ROSTER_UPDATE"); -- we don't need it anymore
			RCLootCouncil:GetGuildOptions() -- get the guild data to the options table now that it's ready
		end
	elseif event == "CHAT_MSG_WHISPER" then
		if itemRunning and isMasterLooter and db.acceptWhispers then
			local msg, sender = ...
			RCLootCouncil:GetItemsFromWhisper(msg, sender)
		end
	end
end


-------- OnCommReceived -------------------
-- Handles communications
-------------------------------------------
function RCLootCouncil:OnCommReceived(prefix, msg, distri, sender)
--[[List of commands:
	start (i)			- ML initiates new looting (number from lootTable we're at)
	lootTable (table)	- ML sends the loot table
	stop				- ML stops looting
	council (table)		- ML sends the council
	mlDB (table)		- ML sends his options database
	vote (name)			- anyone sends a vote
	add (table)			- anyone sends an response on the "roll"
	remove (name)		- ML removes an entry
	verTest	(string)	- anyone sends a version test
	verTestReply (table)- anyone sends a verTest reply
	award(name, table)	- ML sends an award history entrence
--]]
	if prefix == "RCLootCouncil" then
		if sender == GetUnitName("player") then return end; -- don't do anything if we send the message
		local cmd, object = strsplit(" ", msg, 2) -- split the command from the object
		if cmd and object then
			self:debugS("Comm received, cmd: "..cmd..", object: "..object..", sender: "..sender)
		else 
			self:debugS("Comm received, cmd: "..cmd..", sender: "..sender)
		end
		if cmd == 'start' and (isCouncil or isMasterLooter) then
			if not isMasterLooter or nnp then
				RCLootCouncil_Mainframe.abortLooting() -- start with aborting, just in case the masterlooter disconnects during looting
				isRunning = true; -- the ML said start, so we're running
				self:debug("isRunning = true")
				lootNum = tonumber(object);
				RCLootCouncil_initiateLoot(lootTable[lootNum], sender)
			else
				self:debug("A non ML called for a start!")
			end
		elseif cmd == "lootTable" then
			if not isMasterLooter or nnp then
				local test, c = self:Deserialize(object)
				if test then
					for i = 1, #entryTable do -- clear the entryTable for already recieved answers
						wipe(entryTable[i])
					end
					for i = 1, #c do
						GetItemInfo(c[i]) -- query the item info - hopefully they'll be ready for when we need them
					end
					lootTable = c
					RCLootCouncil_LootFrame.setupLoot(lootTable)
				else
					self:debug("Deserialization on lootTable failed!")
				end
			else
				self:debug("A non ML send a lootTable!")
			end
		elseif cmd == "stop" then
			if not isMasterLooter or nnp then
				isRunning = false; -- the ML said stop, so better stop the addon just in case
				self:debug("isRunning = false")
				self:Print("The Master Looter stopped the voting")
				RCLootCouncil_Mainframe.abortLooting()
				RCLootCouncil_Mainframe.stopLooting()
			else
				self:debug("A non ML called for a stop!")
			end
		elseif cmd == 'council' then
			if not isMasterLooter then
				local test, c = self:Deserialize(object)
				if test then
					currentCouncil = c
					isCouncil = RCLootCouncil_Mainframe.isCouncil()
				else
					--TODO: perhaps add a callback for another send
					self:debug("Deserialization on counciltable failed!")
				end
			else
				self:debug("A non ML send a councillist!")
			end

		elseif cmd == 'mlDB' then
			if not isMasterLooter then
				local test, c = self:Deserialize(object)
				if test then
					mlDB = c
				else
					--TODO: perhaps add a callback for another send
					self:debug("Deserialization on mlDB failed!")
				end
			else
				self:debug("A non ML send a mlDB!")
			end
		elseif cmd == 'vote' and (isCouncil or isMasterLooter) then
			for i = 1, #currentCouncil do -- make sure we only accept votes from council members
				if currentCouncil[i] == sender or masterLooter == sender then
					local _, x = self:Deserialize(object)
					local name, devote = strsplit(" ", x, 2)
					RCLootCouncil_Mainframe.voteOther(name, devote, sender)
					return;
				end
			end
			self:Print("Rejected a vote from a non councilmember ("..sender..")")

		elseif cmd == 'add' and (isCouncil or isMasterLooter) then
			local test, c = self:Deserialize(object)
				if test then
					tinsert(entryTable[c["i"]], c[1])
					RCLootCouncil_Mainframe.Update(true)
				else
					--TODO: Add a callback for another send
					self:debug("Deserialization on add failed!")
				end
		elseif cmd == 'remove' and (isCouncil or isMasterLooter) then
			if not isMasterLooter then
				RCLootCouncil_Mainframe.removeEntry(tostring(object))
			else
				self:debug("A non ML ("..sender..") send a remove cmd")
			end
		elseif cmd == 'verTest' then
			self:debug("verTest received from: "..sender)
			if version < object and not hasVerCheck then -- if we're outdated and haven't already showed it to the user
				self:Print("Your version: "..version.." is outdated, newer version is: "..object)
				hasVerCheck = true;				
			end
			local _, class = UnitClass("player")
			self:SendCommMessage("RCLootCouncil", "verTestReply "..self:Serialize({class, guildRank, version}) , "WHISPER", sender)

		elseif cmd == 'verTestReply' then
			local test, c = self:Deserialize(object)
			if test then			
				if version < c[3] and not hasVerCheck then -- if we're outdated
					self:Print("Your version: "..version.." is outdated, newest version is: "..c[3])
					hasVerCheck = true;
				end
				self:debug(""..sender.." uses "..c[3])
				tinsert(c, 2, sender) -- insert sender to the table
				RCLootCouncil_VersionFrame:AddPlayer(c) -- and add the data to the versionFrame							
			else -- if old version
				self:debug(""..sender.." uses "..object)
				RCLootCouncil_VersionFrame:AddPlayer({nil, sender, "Unknown", object})
			end

		elseif cmd == "award" and db.trackAwards and (isCouncil or isMasterLooter) then
			local name, table = string.split(" ", object,2)
			if name and table then -- just to be sure
				local test, c = self:Deserialize(table)
				if test then
					if lootDB[name] then -- if the name is already registered in the table
						tinsert(lootDB[name], c)
					else -- if it isn't
						lootDB[name] = {c};
					end
				else
					self:debug("Deserialization on award failed!")
				end
			else
				self:debug("Couldn't get name or table in award cmd")
			end
		else
			self:debug("Bad command")
		end
	end
end


--------------- debug ---------------------
-- Prints debug messeges if turned on
-------------------------------------------
function RCLootCouncil:debug(msg)
	if debug then 
		self:Print("debug: "..msg)
	end
end

-------------- superDebug -----------------
-- Prints superDebug messages
-------------------------------------------
function RCLootCouncil:debugS(msg)
	if superDebug then
		self:Print("debugS: "..msg)
	end
end

-------------- showMainFrame --------------
-- Shows the Main Frame
-------------------------------------------
function RCLootCouncil_Mainframe.showMainFrame()
	if itemRunning then -- would hide the entries if a councilmember closes and trys to re open it
		RCLootCouncil_Mainframe.Update(true)
	else -- no items, so we want to hide it
		RCLootCouncil_Mainframe.Update(false)
	end
	MainFrame:Show()
end

-------------- hideMainFrame --------------
-- Hides the Main Frame
-------------------------------------------
function RCLootCouncil_Mainframe.hideMainFrame()
	MainFrame:Hide()
end


--------- CloseButton_OnClick -------------
-- When close button is clicked
-------------------------------------------
function CloseButton_OnClick()
	if not isMasterLooter then -- hide if we're not the masterlooter
		RCLootCouncil_Mainframe.hideMainFrame()
	elseif not itemRunning then -- if we are, only hide when nothings running
		RCLootCouncil_Mainframe.hideMainFrame()
	else -- else show confirmation box
		StaticPopup_Show("RCLOOTCOUNCIL_CONFIRM_ABORT")
	end
end

--------- removeBtOnClick ------------------
-- When the remove button is clicked
--------------------------------------------
function RCLootCouncil_Mainframe.removeBtOnClick()
	if selection[1] then
		self:SendCommMessage("RCLootCouncil", "remove "..selection[1], "RAID")
		RCLootCouncil_Mainframe.removeEntry(selection[1])
	else
		self:Print("Couldn't remove the player")
	end
end
--------- awardBtOnClick -------------------
-- When the award button is clicked
--------------------------------------------
function RCLootCouncil_Mainframe.awardBtOnClick()
	StaticPopup_Show("RCLOOTCOUNCIL_CONFIRM_AWARD", itemRunning, selection[1])
end

------------- abortLooting --------------------
-- Aborts the current looting session
-----------------------------------------------
function RCLootCouncil_Mainframe.abortLooting()
	itemRunning = nil;
	CurrentItemTexture:Hide();
	CurrentItemLabel:SetText(" ");
	CurrentItemLvl:SetText(" ");
	CurrentItemType:SetText(" ");
	CurrentItemHover:Hide();
	BtClose:SetText("Close");
	EmptyTexture:Show();
	self:debug("Looting aborted");
	RCLootCouncil_Mainframe.updateSelection(0, true);
	RCLootCouncil_Mainframe.Update(false);
	PeopleToRollLabel:Hide()
	PeopleToRollString:Hide()
	
end

function RCLootCouncil_Mainframe.stopLooting()
	-- clear the entry table efficiently, but keep some of it, as its needed later on.
	for i = 1, #entryTable do
		wipe(entryTable[i])
	end
	lootNum = 0
	lootTable = {}
	itemsToLootIndex = {}
	RCLootCouncil_LootFrame.hide()
	RCLootCouncil_Mainframe.Update(false);
end
------------- ChatCommand ---------------------
-- Handles all the chat commands
-----------------------------------------------
function RCLootCouncil:ChatCommand(msg)
		local input, arg = string.split(" ", msg,2); -- Separates the command from the rest
		input = input:lower(); -- Lower case command

    if not input or input:trim() == "" or input == 'help' then
		self:Print(" ver. " .. version)
		self:Print("Available commands:")
		self:Print("prefix: /rclc or /rc")
		self:Print("- config - Open the options frame")
		self:debug("- debug or d - Toggle debugging")
		self:Print("- show - Shows the main loot frame")
		self:Print("- hide - Hides the main loot frame")
		self:Print("- council - displays the current council")
		self:Print("- councilAdd (name) or cAdd - adds a new member to the loot council")
		self:Print("- remove (name) - removes a player from the loot council")
		self:Print("- deleteEntireCouncil - deletes the entire council")
		self:debug("- test (cmd (object)) - emulate an received comm")
		self:Print("- version - open the Version Checker")
		--self:Print("- history - open the Loot History")

	elseif input == 'config' then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    elseif input == 'debug' or input == 'd' then
		debug = not debug
		if debug then self:Print("Debugging enabled")
		else self:Print("Debugging disabled")
		end

	elseif input == 'show' then
		if not IsInRaid() or isCouncil or isMasterLooter or nnp or not isRunning then -- only the right people may see the window during a raid since they otherwise could watch the entire voting
			RCLootCouncil_Mainframe.showMainFrame()
		end

	elseif input == 'hide' then
		RCLootCouncil_Mainframe.hideMainFrame()
	
	elseif input == 'council' then
		if db.council then
			self:Print("Council: ")
			for _,t in ipairs(db.council) do print(t) end
		else
			self:Print("No council exists")
		end

	elseif input == 'counciladd' or input == 'cadd' then
		if arg then
			tinsert(db.council, arg)
			self:Print(""..arg.." was added to the council!")
		else
			self:Print("Please provide a playername")
		end

	elseif input == 'remove' then
		if arg then
			if db.council then
				for k,v in ipairs(db.council) do
					if v == arg then
						tremove(db.council, k)
						self:Print(""..arg.." was removed from the council")
					end
				end
			end
		end
			
	elseif input == 'deleteentirecouncil' then
		db.council = {}
		self:Print("Council wiped")

	elseif input == 'test' and nnp then
		--self:SendCommMessage("RCLootCouncil", arg, "WHISPER", GetUnitName("player"))
		for k,v in pairs(entryTable) do
			if type(v) == "table" then
				for a,b in pairs(v) do
					if type(b) == "table" then
						for x,y in pairs(b) do
							if type(y) == "table" then
								for i,j in pairs(y) do
									self:debug("entryTable["..k.."]["..a.."]["..x.."]["..i.."]: "..tostring(j))
								end
							else
								self:debug("entryTable["..k.."]["..a.."]["..x.."]: "..tostring(y))
							end
						end
					else
						self:debug("entryTable["..k.."]["..a.."]: "..tostring(b))
					end
				end
			else
				self:debug("entryTable["..k.."]: "..tostring(v))
			end
		end
		for i = 1, #(lootDB[UnitName("player")]) do
		for k,v in pairs(lootDB[UnitName("player")][i]) do self:debug("lootDB[Potdisc]["..i.."]["..k.."]: "..v) end
		end
	elseif input == 'version' then
		self:EnableModule("RCLootCouncil_VersionFrame")
	--elseif input == "history" then
	--	self:EnableModule("RCLootHistory")
	elseif input == "nnp" then
		nnp = not nnp
	elseif input == "debugs" and nnp then
		superDebug = not superDebug
    else
        RCLootCouncil:ChatCommand('help')
    end
end 

---------- initiateLoot --------------
-- Start looting an item
-- (debug use /rc add (item) to manually do it
--------------------------------------
function RCLootCouncil_initiateLoot(item, player)
	if isRunning or nnp then -- only do stuff if we're using the addon or debugging
		if item == nil then -- Be certain we got something here
			self:Print("Cannot initiate loot on a non-item!")
			return;
		end
		if itemRunning then -- if we're already trying to handle an item
			self:Print("Can't start a new loot session while an item is being considered");
			self:debug(item);
			return;
		else -- we're not running an item
			itemRunning = item;
			masterLooter = RCLootCouncil_Mainframe.getML()
			isCouncil = RCLootCouncil_Mainframe.isCouncil()
			if isMasterLooter then -- if we're the ML
				-- create the table of in-raid-councilmembers to send to the concilors
				for _, v in ipairs(db.council) do
					if UnitInRaid(v) then
						tinsert(currentCouncil, v)
					end
				end
				self:SendCommMessage("RCLootCouncil", "council "..self:Serialize(currentCouncil), "RAID") -- let the members know if they're councilmen
				self:debug("Added item: "..item)			
				BtClose:SetText("Cancel Looting")
				RCLootCouncil_Mainframe.prepareLootFrame(item)
				self:SendCommMessage("RCLootCouncil", "start "..lootNum, "RAID") -- tell the council we've started
			elseif isCouncil then -- if we're a member of the council
				self:debug("Added item (council): "..item)
				RCLootCouncil_Mainframe.prepareLootFrame(item)
			end
		end
	end
end

---------- prepareLootFrame ----------
-- Prepares the loot frame for an item
--------------------------------------
function RCLootCouncil_Mainframe.prepareLootFrame(item)
	if not itemRunning then -- double check ;)
		self:debug("prepareLootFrame called without any item running!")
		return
	end
	GetItemInfo(item); -- query up the item
	local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount, thisItemEquipLoc, thisItemTexture = GetItemInfo(item); -- Get the item info
	if thisItemTexture then
		CurrentItemTexture:SetTexture(thisItemTexture); -- Set the texture of the icon box
	else
		CurrentItemTexture:SetTexture("Interface\InventoryItems\WoWUnknownItem01");
	end
	
	CurrentItemTexture:Show(); -- Open up the icon box
	EmptyTexture:Hide(); -- Hide the empty texture

	if iLevel then
		CurrentItemLvl:SetText("ilvl: "..iLevel); -- Show the Item Level
	else
		CurrentItemLvl:SetText("");
	end
	
	if sLink then
		CurrentItemLabel:SetText(sLink); -- Set the item link for color and such
		CurrentItemLabel:SetFont("Fonts\\FRIZQT__.TTF", 16);
	else
		CurrentItemLabel:SetText("LOADING");
	end

	if sSubType and sSubType ~= "Miscellaneous" and thisItemEquipLoc ~= "" then
		CurrentItemType:SetText(getglobal(thisItemEquipLoc)..", "..sSubType); -- getGlobal to translate from global constant to localized name
	elseif sSubType ~= "Miscellaneous" then
		CurrentItemType:SetText(sSubType)
	else
		CurrentItemType:SetText(getglobal(thisItemEquipLoc));
	end

	CurrentItemHover:Show(); -- Make sure we can hover the item
	MasterlooterLabel:SetText(masterLooter);

	PeopleToRollString:Show()
	PeopleToRollLabel:SetText(GetNumGroupMembers()) -- set the amount of people missing the rolling
	PeopleToRollLabel:Show()

	RCLootCouncil_Mainframe.showMainFrame()	
end

------------- translateToSlot ---------------------------------
-- Switches itemEquipLoc to its corresponding slot number
---------------------------------------------------------------
function RCLootCouncil_translateToSlot(itemEquipLoc)
	if itemEquipLoc == "" then return 0
	elseif itemEquipLoc == "INVTYPE_AMMO" then return 0
	elseif itemEquipLoc == "INVTYPE_HEAD" then return 1
	elseif itemEquipLoc == "INVTYPE_NECK" then return 2
	elseif itemEquipLoc == "INVTYPE_SHOULDER" then return 3
	elseif itemEquipLoc == "INVTYPE_BODY" then return 4
	elseif itemEquipLoc == "INVTYPE_CHEST" then return 5
	elseif itemEquipLoc == "INVTYPE_ROBE" then return 5
	elseif itemEquipLoc == "INVTYPE_WAIST" then return 6
	elseif itemEquipLoc == "INVTYPE_LEGS" then return 7
	elseif itemEquipLoc == "INVTYPE_FEET" then return 8
	elseif itemEquipLoc == "INVTYPE_WRIST" then return 9
	elseif itemEquipLoc == "INVTYPE_HAND" then return 10
	elseif itemEquipLoc == "INVTYPE_FINGER" then return 11
	elseif itemEquipLoc == "INVTYPE_TRINKET" then return 13
	elseif itemEquipLoc == "INVTYPE_CLOAK" then return 15
	elseif itemEquipLoc == "INVTYPE_WEAPON" then return 16
	elseif itemEquipLoc == "INVTYPE_SHIELD" then return 17
	elseif itemEquipLoc == "INVTYPE_2HWEAPON" then return 16
	elseif itemEquipLoc == "INVTYPE_WEAPONMAINHAND" then return 16
	elseif itemEquipLoc == "INVTYPE_WEAPONOFFHAND" then return 17
	elseif itemEquipLoc == "INVTYPE_HOLDABLE" then return 17
	elseif itemEquipLoc == "INVTYPE_RANGED" then return 16 -- 18
	elseif itemEquipLoc == "INVTYPE_THROWN" then return 16 -- 18
	elseif itemEquipLoc == "INVTYPE_RANGEDRIGHT"  then return 16 -- 18
	elseif itemEquipLoc == "INVTYPE_RELIC" then return -1 -- 18 (removed)
	elseif itemEquipLoc == "INVTYPE_TABARD" then return 19
	elseif itemEquipLoc == "INVTYPE_BAG" then return 20
	elseif itemEquipLoc == "INVTYPE_QUIVER" then return 20
	else return -1
	end
end

---------- ShowCurrentItemTooltip -------------
-- Shows item running's Tooltip
-----------------------------------------------
function RCLootCouncil_Mainframe.ShowCurrentItemTooltip()
	if itemRunning then
		GameTooltip:SetOwner(MainFrame, "ANCHOR_CURSOR")
		GameTooltip:SetHyperlink(itemRunning)
		GameTooltip:Show()
	end
end

---------- ShowCurrentGearTooltip -------------
-- Shows the current gear Tooltip
-----------------------------------------------
function RCLootCouncil_Mainframe.ShowCurrentGearTooltip(id)
	GameTooltip:SetOwner(MainFrame, "ANCHOR_CURSOR")
	if id then
		id = id + offset -- handle the offset
		if entryTable[lootNum][id][6] then
			GameTooltip:SetHyperlink(entryTable[lootNum][id][6])
		else
			GameTooltip:SetText("Loading...");
		end
			GameTooltip:Show()
	else
		if selection[6] then
			GameTooltip:SetHyperlink(selection[6])
		else
			GameTooltip:SetText("Loading...");
		end
			GameTooltip:Show()
	end
end

---------- ShowCurrentGear2Tooltip -------------
-- Shows the current gear2 Tooltip
-----------------------------------------------
function RCLootCouncil_Mainframe.ShowCurrentGear2Tooltip(id)
	GameTooltip:SetOwner(MainFrame, "ANCHOR_CURSOR")
	if id then
		id = id + offset -- handle the offset
		if entryTable[lootNum][id][7] ~= "" then
			GameTooltip:SetHyperlink(entryTable[lootNum][id][7])
			GameTooltip:Show()
		end
			
	else
		if selection[7] then
			GameTooltip:SetHyperlink(selection[7])
			GameTooltip:Show()
		end
	end
end

------------- toolMouseLeave ----------------------------------------
-- Removes the tooltip when mouse leaves the area
-----------------------------------------------------------------
function RCLootCouncil_Mainframe.toolMouseLeave()
	GameTooltip:Hide()
end


---------- GetGuildRank -------------------
-- returns the player's guild rank
-------------------------------------------
function RCLootCouncil:GetGuildRank()
	GuildRoster()
	if IsInGuild() then
		_, rank, _ = GetGuildInfo("player");
		self:debug("Found guild rank: "..rank)
		guildEventUnregister = true; -- make sure we unregister the event
		return rank;
	else
		return "Unguilded";
	end
end

---------- setClassIcon -------------------
-- Sets the class-texture to a given class
-------------------------------------------
function RCLootCouncil_Mainframe.setClassIcon(frame, class)
	if frame then
		if class then
			frame:SetTexture("Interface\\GLUES\\CHARACTERCREATE\\UI-CHARACTERCREATE-CLASSES"); -- this is the image containing all class icons
			local coords = CLASS_ICON_TCOORDS[class]; -- get the coordinates of the class icon we want
			frame:SetTexCoord(unpack(coords)); -- cut out the region with our class icon according to coords
		else -- if there's no class 
			frame:SetTexture("Interface/ICONS/INV_Sigil_Thorim.png")
		end
	else
		self:debug("Error creating class icon - no frame!");
	end
end

----------- setCharName -------------------
-- Sets the charName text and color
-------------------------------------------
function RCLootCouncil_Mainframe.setCharName(frame, class, text)
	if frame then -- sanity check
		frame:SetText(text); -- set the text
		local classColor = RAID_CLASS_COLORS[class]; -- get the color code
		if not classColor then
			-- if class not found display epic color.
			classColor = {["r"] = 0.63921568627451, ["g"] = 0.2078431372549, ["b"] = 0.93333333333333, ["a"] = 1.0 }
		else
			classColor.a = 1.0;
		end
		frame:SetTextColor(classColor["r"],classColor["g"],classColor["b"],classColor["a"]); -- set the color
	end
end

---------- setCurrentGear -----------------
-- Sets the entry's gear in the given frame(s)
-------------------------------------------
function RCLootCouncil_Mainframe.setCurrentGear(entry, gear1Frame, gear2Frame, show2)
-- entryTable order: (playerName, rank, role, totalIlvl, response, gear1, gear2, votes, class, color[], haveVoted, voters[])
	local _, _, _, _, _, _, _, _, _, thisItemTexture = GetItemInfo(entry[6]); -- Get the gear1 item info
	gear1Frame:SetTexture(thisItemTexture) -- set the texture

	if show2 then -- If the second frame was provided
		local _, _, _, _, _, _, _, _, _, tThisItemTexture = GetItemInfo(entry[7]); -- Get the gear2 item info
		gear2Frame:Show()
		gear2Frame:SetTexture(tThisItemTexture)
	else -- if not, hide, otherwise it would show a previous gear
		gear2Frame:Hide()
	end
end

--------- getPlayerRole ---------------
-- Returns the player's role or none
---------------------------------------
function RCLootCouncil.getPlayerRole(arg)
	local role;
	if not arg then
		role = UnitGroupRolesAssigned("player");
	else
		role = arg
	end
	if role == "TANK" then return "Tank";
	elseif role == "HEALER" then return "Healer";
	elseif role == "DAMAGER" then return "DPS";
	else return "None";	end
end

----------- handleResponse -----------------
-- Handles the response given by LootFrame
--------------------------------------------
function RCLootCouncil.handleResponse(response, id)
-- entryTable order: (playerName, rank, role, totalIlvl, response, gear1, gear2, votes, class, color[], haveVoted, voters[])
	local _, totalIlvl = GetAverageItemLevel()
	local gear1, gear2 = RCLootCouncil.getCurrentGear(lootTable[id]);
	local _, class = UnitClass("player"); -- non-localized class name
	local toAdd = {
		["i"] = id,
			{
				GetUnitName("player"),
				guildRank,
				RCLootCouncil.getPlayerRole(),
				math.floor(totalIlvl),
				response,
				gear1,
				gear2,
				0,
				class,
				color,
				false,
				{""}
		}
	}
	tinsert(entryTable[id], toAdd[1])
	RCLootCouncil_Mainframe.Update(true);
	self:SendCommMessage("RCLootCouncil", "add "..self:Serialize(toAdd), "RAID") -- tell everyone else to add it
end


--------- getCurrentGear ------------------------
-- Returns the player's gear in the given slot(s)
-------------------------------------------------
function RCLootCouncil.getCurrentGear(item)
	local _, _, _, _, _, _, _, _, thisItemEquipLoc = GetItemInfo(item);
	if thisItemEquipLoc == "INVTYPE_FINGER"  then -- if item is a Ring (11, 12)
		return GetInventoryItemLink("player", 11), GetInventoryItemLink("player", 12)
	elseif thisItemEquipLoc == "INVTYPE_TRINKET" then -- if item is a Trinket (13 ,14)
		return GetInventoryItemLink("player", 13), GetInventoryItemLink("player", 14)
	elseif thisItemEquipLoc == "INVTYPE_WEAPON" or thisItemEquipLoc == "INVTYPE_2HWEAPON" then -- if item is 1or2Hand (16, 17)
		return GetInventoryItemLink("player", 16), GetInventoryItemLink("player", 17)
	else -- if it's any other item
		return GetInventoryItemLink("player", RCLootCouncil_translateToSlot(thisItemEquipLoc)), "";
	end
end 
------------- isSelected ----------------------------------------
-- Tests if they're selected or not.
-----------------------------------------------------------------
function RCLootCouncil_Mainframe.isSelected(id)
	if selection then
		id = id + offset -- handle the offset
		return entryTable[lootNum][id] == selection
	else
		return false
	end
end

------------ updateSelection ----------------
-- Selects or updates a given entry
---------------------------------------------
function RCLootCouncil_Mainframe.updateSelection(id, update)
	self:debugS("updateSelection was called");
	id = id + offset -- handle the offset
	if selection then
		for i = 1, MAX_DISPLAY do -- remove any previous selections
			_G["ContentFrameEntry"..i.."BG"]:Hide()
			selection = nil;
		end
	end
	if not update then -- set the selection if we didn't just want to update
		selection = entryTable[lootNum][id]
	end 
	if selection then -- Check if we should update or user clicked a nonshowing row
		-- Initialize variables
		local sLink, iLevel, thisItemTexture;
		local sLink2, thisItemTexture2;
		if selection[6] then -- if they have the first item link, get its info
			_, sLink, _, iLevel, _, _, _, _, _, thisItemTexture = GetItemInfo(selection[6]);
		end
		if selection[7] then -- if they have 2 item links, then get the second's info
			_, sLink2, _, _, _, _, _, _, _, thisItemTexture2 = GetItemInfo(selection[7]);
		end
		
		if isMasterLooter then -- If we're the master looter show the remove and award button
			BtRemove:Show()
			BtAward:Show()
		else
			BtRemove:Hide()
			BtAward:Hide()
		end
		-- Show the if 1 or 2 items stuff
		if sLink2 then -- if there's two items, show the dual selection and hide the rest
			DualItemTexture1:SetTexture(thisItemTexture)
			DualItemTexture2:SetTexture(thisItemTexture2)
			DualItemLabel1:SetText(sLink)
			DualItemLabel1:SetFont("Fonts\\FRIZQT__.TTF", 14);
			DualItemLabel2:SetText(sLink2)
			DualItemLabel2:SetFont("Fonts\\FRIZQT__.TTF", 14);
			DualItemLabel1:Show()
			DualItemLabel2:Show()	
			DualItemTexture1:Show()
			DualItemTexture2:Show()
			DualItemSelection1:Show()
			DualItemSelection2:Show()					
			CurrentSelectionLabel:Hide()
			CurrentSelectionIlvl:Hide()
			CurrentSelectionTexture:Hide()
			CurrentSelectionHover:Hide()
			
		elseif IsEquippableItem(itemRunning) then -- Only show 1 item
			CurrentSelectionLabel:SetText(sLink)
			CurrentSelectionLabel:SetFont("Fonts\\FRIZQT__.TTF", 16);
			CurrentSelectionIlvl:SetText("ilvl: "..iLevel)
			CurrentSelectionTexture:SetTexture(thisItemTexture)
			CurrentSelectionHover:Show()
			CurrentSelectionLabel:Show()
			CurrentSelectionIlvl:Show()
			CurrentSelectionTexture:Show()		
			DualItemLabel1:Hide()
			DualItemLabel2:Hide()	
			DualItemTexture1:Hide()
			DualItemTexture2:Hide()
			DualItemSelection1:Hide()
			DualItemSelection2:Hide()
		end

		-- show the rest
		MainFrameMoreInfo:Show()
		MainFrameMoreInfoHover:Show()
		BtClear:Show()
	else -- if there's no selection, hide everything
		DualItemLabel1:Hide()
		DualItemLabel2:Hide()	
		DualItemTexture1:Hide()
		DualItemTexture2:Hide()
		DualItemSelection1:Hide()
		DualItemSelection2:Hide()
		CurrentSelectionLabel:Hide()
		CurrentSelectionIlvl:Hide()
		CurrentSelectionTexture:Hide()
		CurrentSelectionHover:Hide()
		BtRemove:Hide()
		BtClear:Hide()
		BtAward:Hide()
		MainFrameMoreInfo:Hide()
		MainFrameMoreInfoHover:Hide()
	end

end

------------ removeEntry ------------------
-- Removes an entry from the list
-------------------------------------------
function RCLootCouncil_Mainframe.removeEntry(name)
	self:debugS("Remove Entry was called");
	local id = nil;
	-- find the index of entryTable belonging to name
	for i = 1, #entryTable[lootNum] do
		if entryTable[lootNum][i][1] == name then
			id = i
			break;
		end
	end
	if id then
		tremove(entryTable[lootNum], id) -- remove the entry
		RCLootCouncil_Mainframe.Update(true); -- update
		if selection and selection[1] == name then
			RCLootCouncil_Mainframe.updateSelection(0, true); -- and clear the selection if we had selected the one to be removed
		end
	else 
		self:debug("Couldn't remove entry")
	end
end

------------ vote -------------------------
-- Adds votes to entrytable
-------------------------------------------
function RCLootCouncil_Mainframe.vote(id)
	if id then
		id = id + offset -- handle the offset
		if entryTable[lootNum][id][11] then -- if unvote
			entryTable[lootNum][id][8] = entryTable[lootNum][id][8] - 1;
			entryTable[lootNum][id][11] = false;
			for k,v in pairs(entryTable[lootNum][id][12]) do -- remove the voter from voters table
				if v == GetUnitName("player") then tremove(entryTable[lootNum][id][12], k); end
			end
			RCLootCouncil_Mainframe.Update(true); -- update the list
			self:SendCommMessage("RCLootCouncil", "vote "..self:Serialize(entryTable[lootNum][id][1].." devote"), "RAID") -- tell everyone who you vote for
		else -- if vote
			if not mlDB.selfVote and GetUnitName("player") == entryTable[lootNum][id][1] then -- test that they may vote for themself
				self:Print("The master looter has turned Vote For Self OFF")
				return;
			end
			for i = 1, #entryTable[lootNum] do -- test that they are not multivoting
				 if entryTable[lootNum][i][11] and not mlDB.multiVote then
					 self:Print("The master looter has turned off multivoting")
					 return;
				 end
			end
			entryTable[lootNum][id][8] = entryTable[lootNum][id][8] + 1;
			entryTable[lootNum][id][11] = true
			tinsert((entryTable[lootNum][id][12]), GetUnitName("player")) -- add the voter to the voters table
			RCLootCouncil_Mainframe.Update(true); -- update the list
			self:SendCommMessage("RCLootCouncil", "vote "..self:Serialize(entryTable[lootNum][id][1].." vote"), "RAID") -- tell everyone who you devote for
		end
	else
		self:debug("False id @.vote")
	end
end

------------- voteOther -----------------
-- Add another player's vote to the entry
-----------------------------------------
function RCLootCouncil_Mainframe.voteOther(name, devote, voter)
	if name then
		for i = 1, #entryTable[lootNum] do -- find the id belonging to the name
			if entryTable[lootNum][i][1] == name then
				if devote == "devote" then
					entryTable[lootNum][i][8] = entryTable[lootNum][i][8] - 1; -- remove the vote
					for k,v in pairs(entryTable[lootNum][i][12]) do -- remove the voter from voters table
						if v == voter then tremove(entryTable[lootNum][i][12], k); end
					end
				else
					entryTable[lootNum][i][8] = entryTable[lootNum][i][8] + 1; -- add the vote
					tinsert((entryTable[lootNum][i][12]), voter) -- add the voter to the voters table
				end
			RCLootCouncil_Mainframe.Update(true); -- update the list
			end
		end
	else
		self:debug("Bad name in voteOther")
	end
end

---------- award ------------------------
-- Gives the item to the selected player
-----------------------------------------
function RCLootCouncil_Mainframe.award()
	if not selection or lootNum == 0 then -- if noone is selected or if there's not a loot in the list
		self:debug("award called without selection or with lootNum = 0")
		return;
	else -- if there are
		for i = 1, GetNumGroupMembers() do
			if GetMasterLootCandidate(itemsToLootIndex[lootNum], i) == selection[1] then
				if #itemsToLootIndex > 0 and lootNum > 0 then -- if there really is something to loot
					local _, _, lootQuantity = GetLootSlotInfo(itemsToLootIndex[lootNum])
					if lootQuantity > 0 then -- be certain there's an item
						GiveMasterLoot(itemsToLootIndex[lootNum], i); -- give the item
						self:debug(""..selection[1].." was award with "..itemRunning);
					else
						self:Print("Couldn't award the loot since it has already been awarded!")
						return;
					end
				else -- the function was called by something else
					self:debug("award called without items!")
					self:Print("Error awarding loot, please restart the looting session.")
					return;
				end
				-- Chat output:
				if db.awardMessageChat1 ~= "NONE" then -- if we want to tell who won
					local message = gsub(db.awardMessageText1, "&p", selection[1])
					message = gsub(message, "&i", itemRunning)
					SendChatMessage(message, db.awardMessageChat1); -- then do it
				end
				if db.awardMessageChat2 ~= "NONE" then -- if the user is posting to 2 channels
					local message = gsub(db.awardMessageText2, "&p", selection[1])
					message = gsub(message, "&i", itemRunning)
					SendChatMessage(message, db.awardMessageChat2);
				end

				-- Item awards history:
				-- The ML sends the history data regardless if he wants to track it himself.
				local instanceName, _, _, difficultyName = GetInstanceInfo()
				-- lootDB format: "name" = {"lootWon", "date (d/m/y)", "time (h:m:s)", "instance", "boss", "votes", "itemReplaced1", "itemReplaced2"}
				local table = {["lootWon"] = itemRunning, ["date"] = date("%d/%m/%y"), ["time"] = date("%H:%M:%S"), ["instance"] = instanceName.." "..difficultyName, ["boss"] = bossName, ["votes"] = selection[8], ["itemReplaced1"] = selection[6], ["itemReplaced2"] = selection[7], ["response"] = buttonsDB[selection[5]]["response"], ["responseID"] = selection[5]}
				self:SendCommMessage("RCLootCouncil", selection[1].." "..self:Serialize(table), "RAID")
				-- Only store the data if the user wants to
				if db.trackAwards then 
					if lootDB[selection[1]] then -- if the name is already registered in the table
						tinsert(lootDB[selection[1]], table)
					else -- if it isn't
						lootDB[selection[1]] = {table};
					end
				end
				
				-- Continue with next item or stop?:
				if lootNum < #itemsToLootIndex then -- if there's more items to loot
					RCLootCouncil_Mainframe.abortLooting() -- stop the current looting
					lootNum = lootNum + 1; 
					RCLootCouncil_initiateLoot(GetLootSlotLink(itemsToLootIndex[lootNum]), GetUnitName("player")); -- start it
				else -- if there's not
					self:SendCommMessage("RCLootCouncil", "stop", "RAID") -- tell the council to stop aswell
					RCLootCouncil_Mainframe.abortLooting()
					RCLootCouncil_Mainframe.stopLooting()
					CloseButton_OnClick(); 
				end
				return;
			end		
		end
	end
	-- if the function haven't returned by now then it means noone was in the raid (i.e. debug) or an error
	if debug and GetNumGroupMembers() == 0 then -- not in group and debugging is on
		self:debugS("lootNum: "..lootNum)
		if #itemsToLootIndex > 0 and lootNum > 0 then
			LootSlot(itemsToLootIndex[lootNum]); -- give to self
		else
			LootSlot(lootNum)
		end
		if db.awardMessageChat1 ~= "NONE" then -- if we want to tell who won
			local message = gsub(db.awardMessageText1, "&p", selection[1])
			message = gsub(message, "&i", itemRunning)
			SendChatMessage(message, db.awardMessageChat1); -- then do it
		end
		if db.awardMessageChat2 ~= "NONE" then -- if the user is posting to 2 channels
			local message = gsub(db.awardMessageText2, "&p", selection[1])
			message = gsub(message, "&i", itemRunning)
			SendChatMessage(message, db.awardMessageChat2);
		end
		if db.trackAwards then
			local instanceName, _, _, difficultyName = GetInstanceInfo()
			local table = {["lootWon"] = itemRunning, ["date"] = date("%d/%m/%y"), ["time"] = date("%H:%M:%S"), ["instance"] = instanceName.." "..difficultyName, ["boss"] = bossName, ["votes"] = selection[8], ["itemReplaced1"] = selection[6], ["itemReplaced2"] = selection[7], ["response"] = buttonsDB[selection[5]]["response"], ["responseID"] = selection[5]}
			if lootDB[selection[1]] then -- if the name is already registered in the table
				tinsert(lootDB[selection[1]], table)
			else -- if it isn't
				lootDB[selection[1]] = {table};
			end
		end
		if lootNum < #itemsToLootIndex then -- if there's more items to loot
			RCLootCouncil_Mainframe.abortLooting()
			lootNum = lootNum + 1; 
			RCLootCouncil_initiateLoot(GetLootSlotLink(itemsToLootIndex[lootNum]), GetUnitName("player")); -- start it
		else
			RCLootCouncil_Mainframe.abortLooting()
			RCLootCouncil_Mainframe.stopLooting()
			CloseButton_OnClick()
		end
	else
		self:Print("Error awarding the loot");	
	end
end

------------- getGuildRankNum --------------------
-- Gets the actual rank number for the player
--------------------------------------------------
function RCLootCouncil_Mainframe.getGuildRankNum(playerName)
	GuildRoster();
	if playerName then
		for ci=1, GetNumGuildMembers(true) do
			local name, rank, rankIndex = GetGuildRosterInfo(ci)
			if name == playerName then
				return rankIndex, rank
			end
		end
	end
	return 11
end

------------- getLowestItemLevel -----------------
-- Used in sorting
-- Returns the lower of 2 item levels
--------------------------------------------------
function RCLootCouncil_Mainframe.getLowestItemLevel(entry)
	if entry then
		if entry[7] and entry[7] ~= "" then
			local _, _, _, itemLevel = GetItemInfo(entry[6])
			local _, _, _, itemLevel2 = GetItemInfo(entry[7])
			if itemLevel >= itemLevel2 then
				return itemLevel2
			else
				return itemLevel
			end			
		elseif entry[6] then
			local _, _, _, itemLevel = GetItemInfo(entry[6])
			return itemLevel
		else
			return -1
		end
	end
	return -1
end
			
------------- sortTable --------------------------
-- Sorts the table when you click on a header
--------------------------------------------------
function RCLootCouncil_Mainframe.sortTable(id)
	if currSortIndex == id then -- if we're already sorting this one
		if sortMethod == "asc" then -- then switch the order
			sortMethod = "desc"
		else
			sortMethod = "asc"
		end
	elseif id then -- if we got a valid id
		currSortIndex = id -- then initialize our sort index
		sortMethod = "asc" -- and the order we're sorting in
	end
-- entryTable order: (playerName, rank, role, totalIlvl, response, gear1, gear2, votes, class, color[], haveVoted, voters[])
	if (id == 0) then -- Char Name sorting (alphabetically)
		table.sort(entryTable[lootNum], function(v1, v2)
			if sortMethod == "desc" then
				return v1 and v1[1] > v2[1]
			else
				return v1 and v1[1] < v2[1]
			end
		end)
		
	elseif (id == 1) then -- Guild Rank sorting (numerically)
		table.sort(entryTable[lootNum], function(v1, v2)
			if sortMethod == "desc" then
				return (v1 and RCLootCouncil_Mainframe.getGuildRankNum(v1[1]) > RCLootCouncil_Mainframe.getGuildRankNum(v2[1]))
			else
				return (v1 and RCLootCouncil_Mainframe.getGuildRankNum(v1[1]) < RCLootCouncil_Mainframe.getGuildRankNum(v2[1]))
			end
		end)
		
	elseif (id == 2) then -- Role sorting (alphabetically)
		table.sort(entryTable[lootNum], function(v1, v2)
			if sortMethod == "desc" then
				return v1 and v1[3] > v2[3]
			else
				return v1 and v1[3] < v2[3]
			end
		end)
		
	elseif (id == 3) then -- Totalilvl sorting (numerically)
		table.sort(entryTable[lootNum], function(v1, v2)
			if sortMethod == "desc" then
				return v1 and v1[4] > v2[4]
			else
				return v1 and v1[4] < v2[4]
			end
		end)
		
	elseif (id == 4) then -- Response sorting (need/greed/minor/pass)
		table.sort(entryTable[lootNum], function(v1, v2)
			if sortMethod == "desc" then
				return v1 and v1[5] > v2[5]
			else
				return v1 and v1[5] < v2[5]
			end				
		end)
		
	elseif (id == 5) then -- Item Level Sorting (lowest item level at the top - largest upgrade so to speak)
		table.sort(entryTable[lootNum], function(v1, v2)
			if sortMethod == "desc" then
				return ((v1 ~= nil) and (v2 == nil or ((RCLootCouncil_Mainframe.getLowestItemLevel(v1) ~= -1) and (RCLootCouncil_Mainframe.getLowestItemLevel(v1) > RCLootCouncil_Mainframe.getLowestItemLevel(v2)))))
			else
				return ((v1 ~= nil) and (v2 == nil or ((RCLootCouncil_Mainframe.getLowestItemLevel(v1) ~= -1) and (RCLootCouncil_Mainframe.getLowestItemLevel(v1) < RCLootCouncil_Mainframe.getLowestItemLevel(v2)))))
			end
		end)
		
	elseif (id == 6) then -- votes sorting (numerically)
		table.sort(entryTable[lootNum], function(v1, v2)
			if sortMethod == "desc" then
				return v1 and v1[8] > v2[8]
			else
				return v1 and v1[8] < v2[8]
			end
		end)
	end
	RCLootCouncil_Mainframe.Update(true)
end

------------- getML -----------------------------------
-- returns the masterlooter or the player if debugging
-- or activates masterlooting
-------------------------------------------------------
function RCLootCouncil_Mainframe.getML()
	if not UnitInRaid("player") and nnp then  -- out of raid and debug on
		isMasterLooter = true;
		return GetUnitName("player")
	end
	local lootMethod, _, MLRaidID = GetLootMethod()
	if lootMethod == 'master' then
		local name = GetRaidRosterInfo(MLRaidID)
		if name == GetUnitName("player") then -- double check :)
			isMasterLooter = true
			self:debug("Masterlooter is: "..name)
			return name;
		elseif name then -- if we're not the ML
			self:debug("Masterlooter is: "..name)
			isMasterLooter = false
			return name
		end
	elseif isRunning and UnitIsGroupLeader("player") then -- if masterlooting isn't on, turn it on, but only if we're running and are the raid leader	
		SetLootMethod("master", GetUnitName("player"))
		self:Print("Looting method changed to \"Master Looter\"")
		isMasterLooter = true
		return GetUnitName("player");
	end
	isMasterLooter = false
	return "Noone"; 
end

------------- isCouncil -------------------------------
-- returns true if the player is in the council
-------------------------------------------------------
function RCLootCouncil_Mainframe.isCouncil()
	if isMasterLooter then return true; end;
	if #currentCouncil > 0 then
		for _, v in ipairs(currentCouncil) do
			if v == GetUnitName("player") then
				self:debug("I am in the council!")
				return true
			end
		end
		return false
	end
end

------------ setRank -----------------------
-- Gets the minimum rank from RCRankFrame
-- and saves the council accordingly
--------------------------------------------
function RCLootCouncil_Mainframe.setRank(rank)
	db.minRank = rank;
	GuildRoster()
	for i = 1, GetNumGuildMembers() do
		local name, _, rankIndex = GetGuildRosterInfo(i) -- get info from all guild members
		if rankIndex + 1 <= db.minRank then -- if the member is the required rank, or above
			table.insert(db.council, name) -- then insert them to the council
		end
	end
end

-------------- Update ------------------
-- Handles the scrollbar visualization
-- and updates the contentFrame
-- arg: true to show, false to hide entries
----------------------------------------
function RCLootCouncil_Mainframe.Update(update)
	self:debugS("GetOffset: "..FauxScrollFrame_GetOffset(ContentFrame))
	if update then -- if the table doesn't exist, we want to hide the entries
		FauxScrollFrame_Update(ContentFrame, #entryTable[lootNum], MAX_DISPLAY, 20, nil, nil, nil, nil, nil, nil, true);
	end
	offset = FauxScrollFrame_GetOffset(ContentFrame)
	for i = 1, MAX_DISPLAY do 
		local line = offset + i;
 
		if update and entryTable[lootNum][line] then -- if there's something at a given entry and we want to update
			local entry = entryTable[lootNum][line]
			-- Start setting all the text(ures):
			RCLootCouncil_Mainframe.setCharName(getglobal("ContentFrameEntry"..i.."CharName"),entry[9], entry[1]) -- set the charName and color
			getglobal("ContentFrameEntry"..i.."Rank"):SetText(entry[2])
			getglobal("ContentFrameEntry"..i.."Role"):SetText(entry[3])
			getglobal("ContentFrameEntry"..i.."Totalilvl"):SetText(entry[4])
			getglobal("ContentFrameEntry"..i.."Response"):SetText(mlDB.buttons[entry[5]]["response"])
			getglobal("ContentFrameEntry"..i.."Votes"):SetText(entry[8])
			RCLootCouncil_Mainframe.setClassIcon(getglobal("ContentFrameEntry"..i.."ClassTexture"), entry[9]); -- set the class-texture
			
			-- set the vote button properly
			getglobal("ContentFrameEntry"..i.."BtVote"):Show()
			if entry[11] then
				getglobal("ContentFrameEntry"..i.."BtVote"):SetText("Unvote")
			else
				getglobal("ContentFrameEntry"..i.."BtVote"):SetText("Vote")
			end

			-- set the colors:
			local color = mlDB.buttons[entry[5]]["color"]
			getglobal("ContentFrameEntry"..i.."Rank"):SetTextColor(color[1],color[2],color[3],color[4])
			getglobal("ContentFrameEntry"..i.."Role"):SetTextColor(color[1],color[2],color[3],color[4])
			getglobal("ContentFrameEntry"..i.."Totalilvl"):SetTextColor(color[1],color[2],color[3],color[4])
			getglobal("ContentFrameEntry"..i.."Response"):SetTextColor(color[1],color[2],color[3],color[4])

			-- Set the item textures
			if entry[7] and entry[7] ~= "" then -- If there's 2 items, send them both
				RCLootCouncil_Mainframe.setCurrentGear(entry, getglobal("ContentFrameEntry"..i.."CurrentGear1"), getglobal("ContentFrameEntry"..i.."CurrentGear2"), true);
			elseif IsEquippableItem(itemRunning) then
				RCLootCouncil_Mainframe.setCurrentGear(entry,getglobal("ContentFrameEntry"..i.."CurrentGear1"),getglobal("ContentFrameEntry"..i.."CurrentGear2"), false);
			end
			getglobal("ContentFrameEntry"..i):Show();

			if #entryTable[lootNum] == GetNumGroupMembers() or (GetNumGroupMembers() == 0 and #entryTable[lootNum] == 1) then -- if everybody has rolled or we're alone
				PeopleToRollLabel:SetText("Everyone have rolled")
				PeopleToRollLabel:SetTextColor(0,1,0,1) -- make the text green
			else -- if someone haven't rolled
				PeopleToRollLabel:SetText(GetNumGroupMembers() - #entryTable[lootNum])
				PeopleToRollLabel:SetTextColor(1,1,1,1) -- make the text white
			end

		else -- if there's nothing at an entry
			getglobal("ContentFrameEntry"..i):Hide();
		end	
	end
end
---------- testFrames ---------------
-- Shows all frames for testing
-------------------------------------
function RCLootCouncil_Mainframe.testFrames()
	for j = 1, 9 do
		local itemLink = GetInventoryItemLink("player", j)
		if itemLink ~= nil then				
			lootTable[1] = itemLink	
			lootNum = 1;
			itemRunning = itemLink
			isMasterLooter = true
			mlDB = db
			masterLooter = GetUnitName("player")
			RCLootCouncil_LootFrame.setupLoot(lootTable)	
			RCLootCouncil_Mainframe.prepareLootFrame(itemLink)
			break;
		end
	end
end
---------- raidTestFrames ---------------
-- Tests the frames
-------------------------------------
function RCLootCouncil_Mainframe.raidTestFrames()
	if IsInRaid() then
		local lootMethod, _, MLRaidID = GetLootMethod()
		if lootMethod == 'master' then -- if master looter is turned on
			local name  = GetRaidRosterInfo(MLRaidID)
			if name == GetUnitName("player") then
				isRunning = true
				for i = 1, 9 do
					local itemLink = GetInventoryItemLink("player", i)
					if itemLink ~= nil then
						mlDB = db -- create the master looter db
						self:SendCommMessage("RCLootCouncil", "mlDB "..self:Serialize(mlDB), "RAID") -- and send it
						lootNum = 1
						lootTable[1] = itemLink
						self:debug("itemLink is: "..itemLink)
						self:SendCommMessage("RCLootCouncil", "lootTable "..self:Serialize(lootTable), "RAID")
						RCLootCouncil_initiateLoot(lootTable[lootNum], GetUnitName("player"))
						RCLootCouncil_LootFrame.setupLoot(lootTable)
						break;
					end
				end
			else
				self:Print("You're not the master looter")
			end
		else
			self:Print("Masterlooting must be turned on.")	
		end
	else
		self:Print("You cannot raidtest without being in a raid.")
	end
end

----------- isRunning -------------
-- Sets isRunning = true/false
-- or returns the state if arg
---------------------------------------
function RCLootCouncil.isRunning()
	isRunning = not isRunning
	if isRunning then
		self:Print("Activated")
	else
		self:Print("Deactivated")
	end
end

---------- toDefault --------------
-- Sets the button/response options back to default
-----------------------------------
function RCLootCouncil:toDefault()
	for i = 1, db.numButtons do
		self.db.profile.buttons[i]["text"] = defaults.profile.buttons[i]["text"]
		for j = 1, 4 do
			self.db.profile.buttons[i]["color"][j] = defaults.profile.buttons[i]["color"][j]
		end
		self.db.profile.buttons[i]["response"] = defaults.profile.buttons[i]["response"]
	end
	self.db.profile.numButtons = defaults.profile.numButtons;
end

--------- GetVersion -------------
-- Returns the current version
----------------------------------
function RCLootCouncil:GetVersion()
	return version;
end

---------- voteHover -------------
-- Displays the voters
----------------------------------
function RCLootCouncil_Mainframe:voteHover(id)
	if (not mlDB.anonymousVoting or (mlDB.masterLooterOnly and masterLooter == GetUnitName("player"))) and #entryTable[lootNum][id][12] > 1 then -- check if voting is anonymous
		if id then 
			id = id + offset
			GameTooltip:SetOwner(MainFrame, "ANCHOR_CURSOR")
			GameTooltip:AddLine("Voters\n")
			for k,v in pairs(entryTable[lootNum][id][12]) do
				GameTooltip:AddLine(v,1,1,1)
			end
			GameTooltip:Show()
		else
		 self:debug("Bad id in voteHover")
		end
	end
end

---------- PeopleToRollHover ----------------
-- Shows which people there's still to roll
---------------------------------------------
function RCLootCouncil_Mainframe:PeopleToRollHover()
	local peopleToRoll = {}	
	for i = 1, GetNumGroupMembers() do
		local name = GetRaidRosterInfo(i)
		local test = true
		for j = 1, #entryTable[lootNum] do
			if entryTable[lootNum][j][1] == name then test = false; end			
		end
		if test then tinsert(peopleToRoll, name); end
	end
	if #peopleToRoll >= 1 then
		GameTooltip:SetOwner(MainFrame, "ANCHOR_CURSOR")
		GameTooltip:AddLine("People still to roll\n")
		for k,v in pairs(peopleToRoll) do
			GameTooltip:AddLine(v,1,1,1)
		end
		GameTooltip:Show();
	end
end

---------- LootOnClick ------------------
-- Hooked from LootButton_OnClick
-----------------------------------------
function RCLootCouncil:LootOnClick(button)	
	if InCombatLockdown() and IsAltKeyDown() then 
		self:Print("Cannot initiate loot while in combat")
	elseif db.altClickLooting and IsAltKeyDown() and not IsShiftKeyDown() and not IsControlKeyDown() then
		self:debug("LootOnClick called")
		-- check that we don't add an item we're already looting
		for k,v in pairs(itemsToLootIndex) do
			if v == button.slot then -- if we're already looting that slot
				self:Print("That slot is already being looted.")
				return;
			end
		end
		for i = 1, #entryTable do -- clear the entryTable for already recieved answers
			wipe(entryTable[i])
		end
		if getglobal("ElvLootFrame") then
			button.slot = button:GetID() -- ElvUI hack
		end

		tinsert(lootTable, GetLootSlotLink(button.slot)) -- add the item link to the table
		tinsert(itemsToLootIndex, button.slot) -- add its index to the lootTable

		self:SendCommMessage("RCLootCouncil", "lootTable "..self:Serialize(lootTable), "RAID")
		
		RCLootCouncil_LootFrame.setupLoot(lootTable)
		if #lootTable == 1 then -- only initiate the first item
			lootNum = 1;
			RCLootCouncil_initiateLoot(GetLootSlotLink(button.slot), GetUnitName("player"))
		end
	end
end

------------- GetItemsFromWhisper ------------
-- Extracts items from a whisper and adds them
-- to the entryFrame.
----------------------------------------------
function RCLootCouncil:GetItemsFromWhisper(msg, sender)
	local theItem = msg:find("|Hitem:"); -- See if they linked an item	
	if theItem  then -- If they entered a valid item
		self:debugS("item1 might be: "..theItem)
		local actualItemString2; -- Initialize for possibility of 2 item links
		local startLoc = string.find(msg, "Hitem:") -- Make sure they linked an item
		local endLoc = string.find(msg, "|", startLoc)
		local actualItemString = string.match(msg, "|%x+|Hitem:.-|h.-|h|r");
		self:debugS("actualItemString: "..actualItemString)		
		theItem = string.find(msg, "Hitem:", endLoc) --See if they linked a second item
			
		if theItem then -- If they did
			self:debugS("Two items found!")
			local laterString = string.sub(msg, endLoc);
			actualItemString2 = string.match(laterString, "|%x+|Hitem:.-|h.-|h|r");
		end
			
		local _, iLink1 = GetItemInfo(actualItemString); -- Get better info for item 1
		local iLink2;
		if actualItemString2 then  _, iLink2 = GetItemInfo(actualItemString2); end -- and item 2
			
		-- now that the items is in place, we need some info on the player, and we need to generate it from only the name
		local name, class, rank, role;
		for i = 1, GetNumGroupMembers() do
			name, _, _, _, _, class = GetRaidRosterInfo(i)
			if name == sender then
				role = UnitGroupRolesAssigned("raid"..i)
				role = RCLootCouncil.getPlayerRole(role)
				_, rank = GetGuildInfo(sender)
				break;
			end
		end
		-- add the entry to the player's own entryTable
		local toAdd = {
			["i"] = lootNum,
			{
				sender,
				rank or "",
				role or "",
				0,
				1,
				iLink1,
				iLink2 or "",
				0,
				class,
				color,
				false,
				{""}
			}
		}
		tinsert(entryTable[lootNum], toAdd[1])
		self:SendCommMessage("RCLootCouncil", "add "..self:Serialize(toAdd), "RAID")	
		RCLootCouncil_Mainframe.Update(true)
	end
end

------------ MoreInfoHover -----------
-- Creates the tooltip for MoreInfo
--------------------------------------
function RCLootCouncil_Mainframe:MoreInfoHover()
	GameTooltip:SetOwner(MainFrame, "ANCHOR_CURSOR")
	local name = selection[1]
	GameTooltip:AddLine(name) -- add the name as the first line
	GameTooltip:AddLine(" ")
	-- find the last awarded mainspec item and date
	if lootDB[name] then -- if they're even in the db
		for i = #lootDB[name], 1, -1 do -- start with the end, and count downwards
			if lootDB[name][i]["responseID"] == 1 then -- the last mainspec loot awarded
				local dateString = RCLootCouncil:GetNumberOfDaysFromNow(lootDB[name][i]["date"])
				local firstDateString = RCLootCouncil:GetNumberOfDaysFromNow(lootDB[name][1]["date"])
				GameTooltip:AddDoubleLine("Time since last MainSpec loot:", dateString, 1,1,1, 1,1,1)
				GameTooltip:AddDoubleLine("Loot:",							lootDB[name][i]["lootWon"], 1,1,1, 1,1,1)
				GameTooltip:AddDoubleLine("Dropped by:",					lootDB[name][i]["boss"], 1,1,1, 0.862745, 0.0784314, 0.235294)
				GameTooltip:AddDoubleLine("From:",							lootDB[name][i]["instance"], 1,1,1, 0.823529, 0.411765, 0.117647)
				GameTooltip:AddDoubleLine("Item(s) replaced:",				lootDB[name][i]["itemReplaced1"], 1,1,1, 1,1,1)
				if lootDB[name][i]["itemReplaced2"] ~= "" then
					GameTooltip:AddDoubleLine(" ",				lootDB[name][i]["itemReplaced2"], 1,1,1, 1,1,1)
				end
				local itemsReceivedToday = {}
				for k = #lootDB[name], 1, -1 do
					if lootDB[name][k]["date"] == date("%d/%m/%y") then -- any loots today?
						tinsert(itemsReceivedToday, lootDB[name][k]["lootWon"])
					end
				end
				if #itemsReceivedToday > 0 then
					GameTooltip:AddLine(" ")
					GameTooltip:AddDoubleLine("Item(s) received today:",	itemsReceivedToday[1], 1,1,1, 1,1,1)
					for k = 2, #itemsReceivedToday do
						GameTooltip:AddDoubleLine(" ",	itemsReceivedToday[k], 1,1,1, 1,1,1)
					end
				end
			
				GameTooltip:AddLine(" ")
				GameTooltip:AddLine(name.." has received a total of "..#lootDB[name].." items over "..firstDateString..".", 1,1,1)
				GameTooltip:Show()
				return;
			end
		end
		-- if it get here, it means, that they've recieved loot, but never mainspec
		GameTooltip:AddLine("Has recieved a total of "..#lootDB[name].." items, but not a single one needed for mainspec.", 1,1,1)
		GameTooltip:Show()
	else -- if they're not
		GameTooltip:AddLine("Has not been logged getting any loot!", 1,1,1)
		GameTooltip:AddLine("Best give 'em some :)", 1,1,1)
		GameTooltip:Show()
	end
end

------------------ GetNumberOfDaysFromNow ---------------------
-- Calculates the number of days and years from today to arg.
-- Returns a formatted string for use in MoreInfoHover.
---------------------------------------------------------------
function RCLootCouncil:GetNumberOfDaysFromNow(date)
	local d, m, y = strsplit("/", date, 3)
	local cDayNumber = time()
	local dayNumber = time({year = "20"..y, month = m, day = d})
	local secondsBetween = cDayNumber - dayNumber;
	local days = (secondsBetween / 3600) /24; -- recalculate from seconds to days
	if days < 1 then
		return "today";
	elseif days > 365 then
		local years = 1;
		days = days / 365
		for i = 1, 1000 do -- "infinity" loop
			if days > 365 then -- needs more years subtracted
				days = days / 365
				years = years + 1
			else
				return floor(days).." days and "..years.." years";
			end
		end
	else
		return floor(days).." days";
	end
end

------------ GetVariable ---------
-- Returns the variable from string arg
----------------------------------
function RCLootCouncil:GetVariable(arg)
	if arg == "isCouncil" then
		return isCouncil;
	elseif arg == "isMasterLooter" then
		return isMasterLooter;
	elseif arg == "isRunning" then
		return isRunning;
	elseif arg == "mlDB" then
		if mlDB then return mlDB; else return db; end;
	end
end

------------ LootBoE -------------------------------
-- Checks if an item is BoE and if we should loot it
----------------------------------------------------
function RCLootCouncil:LootBoE(item)
	if not item then return false; end
	GameTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	GameTooltip:SetHyperlink(item)
	if GameTooltip:NumLines() > 1 then -- check that there is something here
		for i = 1, 4 do -- BoE status won't be further away than line 4
			local line = _G['GameTooltipTextLeft' .. i]
			if line and line.GetText then
				if line:GetText() == ITEM_BIND_ON_EQUIP then
					GameTooltip:Hide()
					if db.autolootBoE then return true; else return false; end
				end
			end
		end
	end
	GameTooltip:Hide()
	return true; -- it's not a BoE
end